/////////////////////////////////////////////////////////////////////////////
// Weed ccorrect plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script ccorrect.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////

#define _UNIQUE_ID_ "0XED963E5BC8CD926B"

#define NEED_PALETTE_UTILS

#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////

#include <stdio.h>

static int verbosity = WEED_VERBOSITY_ERROR;

typedef struct {
  double ored, ogreen, oblue ;
  unsigned char r[256], g[256], b[256] ;
} sdata_t;

enum {
  P_red,
  P_green,
  P_blue,
};

static void make_table(unsigned char *tab, double val) {
  for (int i = 0; i < 256; i++) {
    int ival = (val * i + .5);
    tab[i] = ival > 255 ? (unsigned char)255 : (unsigned char)ival;
  }
}

/////////////////////////////////////////////////////////////////////////////

static weed_error_t ccorrect_init(weed_plant_t *inst) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);
  sdata_t *sdata;

  double red = weed_param_get_value_double(in_params[P_red]);
  double green = weed_param_get_value_double(in_params[P_green]);
  double blue = weed_param_get_value_double(in_params[P_blue]);
  weed_free(in_params);

  if (!(weed_instance_get_flags(inst) & WEED_INSTANCE_UPDATE_GUI_ONLY)) {
    sdata = (sdata_t *)weed_calloc(1, sizeof(sdata_t));
    if (!sdata) return WEED_ERROR_MEMORY_ALLOCATION;

    if (1) {
      for (int i = 0; i < 256; i++) {
        sdata->r[i] = sdata->g[i] = sdata->b[i] = 0;
      }
      sdata->ored = sdata->ogreen = sdata->oblue = 0.;
    }
    weed_set_instance_data(inst, sdata);
  }
  else sdata = weed_get_instance_data(inst, sdata);

  return WEED_SUCCESS;
}


static weed_error_t ccorrect_deinit(weed_plant_t *inst) {
  sdata_t *sdata = weed_get_instance_data(inst, sdata);

  if (sdata) weed_free(sdata);
  weed_set_instance_data(inst, NULL);

  return WEED_SUCCESS;
}


static weed_error_t ccorrect_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  unsigned char *src = weed_channel_get_pixel_data(in_chan);
  unsigned char *dst = weed_channel_get_pixel_data(out_chan);
  int inplace = (src == dst);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int iheight = weed_channel_get_height(in_chan);
  int is_threading = weed_is_threading(inst);
  int offset = 0;
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);
  sdata_t *sdata = weed_get_instance_data(inst, sdata);
  if (!sdata) return WEED_ERROR_REINIT_NEEDED;
  else {
    double red = weed_param_get_value_double(in_params[P_red]);
    double green = weed_param_get_value_double(in_params[P_green]);
    double blue = weed_param_get_value_double(in_params[P_blue]);
    weed_free(in_params);

    if (is_threading) {
      offset = weed_channel_get_offset(out_chan);
      src += offset * irow;
      dst += offset * orow;
    }

    if (1) {
      int offs = 0;

      if (red != sdata->ored) {
        make_table(sdata->r, red);
        sdata->ored = red;
      }

      if (green != sdata->ogreen) {
        make_table(sdata->g, green);
        sdata->ogreen = green;
      }

      if (blue != sdata->oblue) {
        make_table(sdata->b, blue);
        sdata->oblue = blue;
      }

      offs = rgb_offset(pal);
      width *= psize;

      for (int j = 0; j < height; j++) {
        for (int i = 0; i < width; i += psize) {
          if (pal == WEED_PALETTE_BGR24 || pal == WEED_PALETTE_BGRA32) {
            dst[j * orow + i] = sdata->b[src[j * irow + i]];
            dst[j * orow + i + 1] = sdata->g[src[j * irow + i + 1]];
            dst[j * orow + i + 2] = sdata->r[src[j * irow + i + 2]];
            if (!inplace && pal == WEED_PALETTE_BGRA32) dst[j * orow + i + 3] = src[j * irow + i + 3];
          } else {
            if (!inplace && pal == WEED_PALETTE_ARGB32) dst[j * orow + i] = src[j * irow + i];
            dst[j * orow + i + offs] = sdata->r[src[j * irow + i + offs]];
            dst[j * orow + i + 1 + offs] = sdata->g[src[j * irow + i + 1 + offs]];
            dst[j * orow + i + 2 + offs] = sdata->b[src[j * irow + i + 2 + offs]];
            if (!inplace && pal == WEED_PALETTE_RGBA32) dst[j * orow + i + 3] = src[j * irow + i + 3];
          }
        }
      }
    }
  }

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  uint64_t unique_id;
  int palette_list[] = ALL_RGB_PALETTES;
  weed_plant_t *in_chantmpls[] = {
      weed_channel_template_init("in_channel0", 0),
      NULL};
  weed_plant_t *out_chantmpls[] = {
      weed_channel_template_init("out_channel0", WEED_CHANNEL_CAN_DO_INPLACE),
      NULL};
  weed_plant_t *in_paramtmpls[] = {
      weed_float_init("red", "_Red factor", 1., 0., 4.),
      weed_float_init("green", "_Green factor", 1., 0., 4.),
      weed_float_init("blue", "_Blue factor", 1., 0., 4.),
      NULL};
  weed_plant_t *pgui;
  int filter_flags = WEED_FILTER_HINT_MAY_THREAD;

  verbosity = weed_get_host_verbosity(host_info);

  pgui = weed_paramtmpl_get_gui(in_paramtmpls[P_red]);
  weed_set_int_value(pgui, WEED_LEAF_DECIMALS, 2);

  pgui = weed_paramtmpl_get_gui(in_paramtmpls[P_green]);
  weed_set_int_value(pgui, WEED_LEAF_DECIMALS, 2);

  pgui = weed_paramtmpl_get_gui(in_paramtmpls[P_blue]);
  weed_set_int_value(pgui, WEED_LEAF_DECIMALS, 2);

  filter_class = weed_filter_class_init("colour correction", "salsaman", 1, filter_flags, palette_list,
    ccorrect_init, ccorrect_process, ccorrect_deinit, in_chantmpls, out_chantmpls, in_paramtmpls, NULL);

  weed_plugin_info_add_filter_class(plugin_info, filter_class);

  if (!sscanf(_UNIQUE_ID_, "0X%lX", &unique_id) || !sscanf(_UNIQUE_ID_, "0x%lx", &unique_id)) {
    weed_set_int64_value(plugin_info, WEED_LEAF_UNIQUE_ID, unique_id);
  }

  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

