/////////////////////////////////////////////////////////////////////////////
// Weed colorkey plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script colorkey.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////

#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////


static int verbosity = WEED_VERBOSITY_ERROR;
enum {
  P_delta,
  P_opac,
  P_col,
};

static weed_error_t colorkey_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t **in_chans = weed_get_in_channels(inst, NULL);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  unsigned char *src0 = weed_channel_get_pixel_data(in_chans[0]);
  unsigned char *src1 = weed_channel_get_pixel_data(in_chans[1]);
  unsigned char *dst = weed_channel_get_pixel_data(out_chan);
  int inplace = (src0 == dst);
  int pal = weed_channel_get_palette(in_chans[0]);
  int irow0 = weed_channel_get_stride(in_chans[0]);
  int irow1 = weed_channel_get_stride(in_chans[1]);
  int iheight = weed_channel_get_height(in_chans[0]);
  int is_threading = weed_is_threading(inst);
  int offset = 0;
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int *iarray;
  int col_red, col_green, col_blue;
  double delta = weed_param_get_value_double(in_params[P_delta]);
  double opac = weed_param_get_value_double(in_params[P_opac]);
  iarray = weed_param_get_array_int(in_params[P_col], NULL);
  col_red = iarray[0];
  col_green = iarray[1];
  col_blue = iarray[2];
  weed_free(iarray);
  weed_free(in_params);

  if (is_threading) {
    offset = weed_channel_get_offset(out_chan);
    src0 += offset * irow0;
    src1 += offset * irow1;
    dst += offset * orow;
  }

  if (1) {
    double opacx, xdelta = delta * 2.;
    int red, green, blue;
    int col_red_min, col_green_min, col_blue_min;
    int col_red_max, col_green_max, col_blue_max;

    delta /= 2.;

    col_red_min = col_red - (int)(col_red * delta + .5);
    col_green_min = col_green - (int)(col_green * xdelta + .5);
    col_blue_min = col_blue - (int)(col_blue * delta + .5);

    xdelta *= 2.;
    delta *= 2.;

    col_red_max = col_red + (int)((255 - col_red) * delta + .5);
    col_green_max = col_green + (int)((255 - col_green) * xdelta + .5);
    col_blue_max = col_blue + (int)((255 - col_blue) * delta + .5);

    width *= 3;

    for (int i = 0; i < height; i++) {
      for (int j = 0; j < width; j += 3) {
        if (pal == WEED_PALETTE_RGB24) {
          red = src0[irow0 * i + j]; green = src0[irow0 * i + j + 1]; blue = src0[irow0 * i + j + 2];
        } else {
          blue = src0[irow0 * i + j]; green = src0[irow0 * i + j + 1]; red = src0[irow0 * i + j + 2];
        }
        if (red >= col_red_min && red <= col_red_max && green >= col_green_min && green <= col_green_max
            && blue >= col_blue_min && blue <= col_blue_max) {
          dst[orow * i + j] = src0[irow0 * i + j] * ((opacx = 1. - opac)) + src1[irow1 * i + j] * opac;
          dst[orow * i + j + 1] = src0[irow0 * i + j + 1] * (opacx) + src1[irow1 * i + j + 1] * opac;
          dst[orow * i + j + 2] = src0[irow0 * i + j + 2] * (opacx) + src1[irow1 * i + j + 2] * opac;
        } else if (!inplace) weed_memcpy(&dst[orow * i + j], &src0[irow0 * i + j], 3);
      }
    }
  }
  weed_free(in_chans);

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  int palette_list[] = {WEED_PALETTE_RGB24, WEED_PALETTE_BGR24, WEED_PALETTE_END};
  weed_plant_t *in_chantmpls[] = {
      weed_channel_template_init("in_channel0", 0),
      weed_channel_template_init("in_channel1", 0),
      NULL};
  weed_plant_t *out_chantmpls[] = {
      weed_channel_template_init("out_channel0", WEED_CHANNEL_CAN_DO_INPLACE),
      NULL};
  weed_plant_t *in_paramtmpls[] = {
      weed_float_init("delta", "_Delta", .2, 0., 1.),
      weed_float_init("opac", "_Opacity", 1., 0., 1.),
      weed_colRGBi_init("col", "_Colour", 0, 0, 255),
      NULL};
  weed_plant_t *pgui;
  int filter_flags = WEED_FILTER_HINT_MAY_THREAD;

  verbosity = weed_get_host_verbosity(host_info);

  pgui = weed_paramtmpl_get_gui(in_paramtmpls[P_delta]);
  weed_set_int_value(pgui, WEED_LEAF_DECIMALS, 2);

  pgui = weed_paramtmpl_get_gui(in_paramtmpls[P_opac]);
  weed_set_int_value(pgui, WEED_LEAF_DECIMALS, 2);

  filter_class = weed_filter_class_init("colour key", "salsaman", 1, filter_flags, palette_list,
    NULL, colorkey_process, NULL, in_chantmpls, out_chantmpls, in_paramtmpls, NULL);

  weed_plugin_info_add_filter_class(plugin_info, filter_class);
  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

