/////////////////////////////////////////////////////////////////////////////
// Weed noise plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script noise.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////
#define NEED_PALETTE_UTILS
#define NEED_RANDOM

#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////


static int verbosity = WEED_VERBOSITY_ERROR;

static weed_error_t noise_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  unsigned char *src = weed_channel_get_pixel_data(in_chan);
  unsigned char *dst = weed_channel_get_pixel_data(out_chan);
  int inplace = (src == dst);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int iheight = weed_channel_get_height(in_chan);
  int is_threading = weed_is_threading(inst);
  int offset = 0;
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);

  if (is_threading) {
    offset = weed_channel_get_offset(out_chan);
    src += offset * irow;
    dst += offset * orow;
  }

  if (1) {
    width *= psize;
    if (psize == 3) {
      for (int j = 0; j < height; j++) {
        for (int i = 0; i < width; i++) {
          dst[j * orow + i] = src[j * irow + i] + ((fastrnd_int64() & 0xF8000000) >> 27) - 16;
        }
      }
    }
    else {
      for (int j = 0; j < height; j++) {
        for (int i = 0; i < width; i += 4) {
          dst[j * orow + i] = src[j * irow + i] + ((fastrnd_int64() & 0xF8000000) >> 27) - 16;
          dst[j * orow + i + 1] = src[j * irow + i + 1] + ((fastrnd_int64() & 0xF8000000) >> 27) - 16;
          dst[j * orow + i + 2] = src[j * irow + i + 2] + ((fastrnd_int64() & 0xF8000000) >> 27) - 16;
          dst[j * orow + i + 3] = src[j * irow + i + 3];
        }
      }
    }
  }

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  int palette_list[] = ALL_RGBX_PALETTES;
  weed_plant_t *in_chantmpls[] = {
      weed_channel_template_init("in_channel0", 0),
      NULL};
  weed_plant_t *out_chantmpls[] = {
      weed_channel_template_init("out_channel0", WEED_CHANNEL_CAN_DO_INPLACE),
      NULL};
  int filter_flags = WEED_FILTER_HINT_MAY_THREAD;

  verbosity = weed_get_host_verbosity(host_info);

  filter_class = weed_filter_class_init("noise", "salsaman", 1, filter_flags, palette_list,
    NULL, noise_process, NULL, in_chantmpls, out_chantmpls, NULL, NULL);

  weed_filter_set_description(filter_class, "A simple filter which overlays random noise on each frame.");

  weed_plugin_info_add_filter_class(plugin_info, filter_class);
  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

