/////////////////////////////////////////////////////////////////////////////
// Weed pan_and_zoom plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script pan_and_zoomy.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////
#define NEED_PALETTE_UTILS
#define NEED_RANDOM

#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////

#include <math.h>

static int verbosity = WEED_VERBOSITY_ERROR;

typedef struct {
  int phase;
  int count;
  volatile int wait;
  volatile int ready;
  volatile int tot;
  double dscale, scale;
  double dx, dy;
  double cx, cy;
  int otrig; ;
} sdata_t;

enum {
  P_trigger,
};

static const double scmax = exp(3.);

/////////////////////////////////////////////////////////////////////////////

static weed_error_t pan_and_zoom_init(weed_plant_t *inst) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);
  sdata_t *sdata;

  int trigger = weed_param_get_value_boolean(in_params[P_trigger]);
  weed_free(in_params);

  if (!(weed_instance_get_flags(inst) & WEED_INSTANCE_UPDATE_GUI_ONLY)) {
    sdata = (sdata_t *)weed_calloc(1, sizeof(sdata_t));
    if (!sdata) return WEED_ERROR_MEMORY_ALLOCATION;

    weed_set_int64_value(inst, WEED_LEAF_PLUGIN_RANDOM_SEED,
        weed_get_int64_value(inst, WEED_LEAF_RANDOM_SEED, NULL));
    weed_set_instance_data(inst, sdata);
  }
  else sdata = weed_get_instance_data(inst, sdata);

  return WEED_SUCCESS;
}


static weed_error_t pan_and_zoom_deinit(weed_plant_t *inst) {
  sdata_t *sdata = weed_get_instance_data(inst, sdata);

  if (sdata) weed_free(sdata);
  weed_set_instance_data(inst, NULL);

  return WEED_SUCCESS;
}


static weed_error_t pan_and_zoom_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  unsigned char *src = weed_channel_get_pixel_data(in_chan);
  unsigned char *dst = weed_channel_get_pixel_data(out_chan);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int iheight = weed_channel_get_height(in_chan);
  int is_threading = weed_is_threading(inst);
  int offset = 0;
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);
  sdata_t *sdata = weed_get_instance_data(inst, sdata);
  if (!sdata) return WEED_ERROR_REINIT_NEEDED;
  else {
    int trigger = weed_param_get_value_boolean(in_params[P_trigger]);
    weed_free(in_params);

    if (is_threading) {
      offset = weed_channel_get_offset(out_chan);
      src += offset * irow;
      dst += offset * orow;
    }

    if (1) {
      int offsx, offsy, widthx, dx, dy, tt;
      double fscalex;

      if (is_threading) {
        if (!weed_plant_has_leaf(inst, WEED_LEAF_STATE_UPDATED)) {
          return WEED_ERROR_FILTER_INVALID;
        }
        /* host should set this to WEED_FALSE for the first thread, then wait for WEED_TRUE */
        if (weed_get_boolean_value(inst, WEED_LEAF_STATE_UPDATED, NULL) == WEED_FALSE) {
          if (trigger && !sdata->otrig) sdata->count = 0;
          sdata->otrig = trigger;
          if (sdata->count <= 0) {
            float tscale, tx, ty;
            double dcount = 0.;
            if (sdata->phase > 0) {
              sdata->count = 8 + fastrand_int_re(32, inst, WEED_LEAF_PLUGIN_RANDOM_SEED);
              dcount = (double)sdata->count;
            }
            switch (sdata->phase++) {
            case 4:
              sdata->phase = 0;
            case 0:
              sdata->cx = sdata->cy = 0.5;
              sdata->scale = 1;
              sdata->count = 0;
              break;
            case 1:
              // pick target scale
              tscale = fastrnd_dbl(3.);
              sdata->dscale = (1. + tscale) / dcount;
              //tx = (fastrnd_dbl(1.) - .5);// / sdata->scale;
              //ty = (fastrnd_dbl(1.) - .5);// / sdata->scale;
              //sdata->dx = tx / dcount;
              //sdata->dy = ty / dcount;
              break;
            case 2:
              //sdata->count = 0;
              // pick target x, y
              tx = (fastrnd_dbl(1.) - .5);// / sdata->scale;
              ty = (fastrnd_dbl(1.) - .5);// / sdata->scale;
              sdata->dx = tx / dcount;
              sdata->dy = ty / dcount;
              sdata->dscale = 0;
              break;
            case 3:
              // return to base
              sdata->dx = (.5 - sdata->cx) / dcount;
              sdata->dy = (.5 - sdata->cy) / dcount;
              sdata->dscale = (1. - sdata->scale) / dcount;
              break;
            default:
              break;
            }
          }

          sdata->scale += sdata->dscale;
          sdata->cx += sdata->dx / sdata->scale;
          sdata->cy += sdata->dy / sdata->scale;

          if (sdata->scale > scmax) sdata->scale = scmax;
          if (sdata->scale < 1.) sdata->scale = 1.;

          if (sdata->cx < .5 / sdata->scale) sdata->cx = .5 / sdata->scale;
          if (sdata->cx > 1. - .5 / sdata->scale) sdata->cx = 1. - .5 / sdata->scale;
          if (sdata->cy < .5 / sdata->scale) sdata->cy = .5 / sdata->scale;
          if (sdata->cy > 1. - .5 / sdata->scale) sdata->cy = 1. - .5 / sdata->scale;

          sdata->count--;
          weed_set_boolean_value(inst, WEED_LEAF_STATE_UPDATED, WEED_TRUE);
        }
      }

      offsx = sdata->cx * width;
      offsy = sdata->cy * iheight;
      src -= offset * irow;

      widthx = width * psize;
      fscalex = sdata->scale * psize;
      height += offset;

      for (int y = offset; y < height; y++) {
        dy = (int)(offsy + (float)(y - iheight / 2) / sdata->scale + .5);
        if (dy >= iheight) break;
        if (dy < 0) continue;
        for (int x = 0; x < widthx; x += psize) {
          dx = (int)(offsx + (float)(x - widthx / 2) / fscalex + .5);
          if (dx >= width) break;
          if (dx < 0) continue;
          weed_memcpy(&dst[(y - offset) * orow + x],
            &src[irow * dy + dx * psize], psize);
        }
      }
    }
  }

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  int palette_list[] = ALL_PACKED_PALETTES;
  weed_plant_t *in_chantmpls[] = {
      weed_channel_template_init("in_channel0", 0),
      NULL};
  weed_plant_t *out_chantmpls[] = {
      weed_channel_template_init("out_channel0", 0),
      NULL};
  weed_plant_t *in_paramtmpls[] = {
      weed_switch_init("trigger", "_Trigger", WEED_FALSE),
      NULL};
  weed_plant_t *gui;
  char *rfx_strings[] = {
      "special|ignored|1|",
      NULL};
  int filter_flags = WEED_FILTER_HINT_MAY_THREAD | WEED_FILTER_HINT_STATEFUL;

  verbosity = weed_get_host_verbosity(host_info);

  filter_class = weed_filter_class_init("pan and zoom", "salsaman", 1, filter_flags, palette_list,
    pan_and_zoom_init, pan_and_zoom_process, pan_and_zoom_deinit, in_chantmpls, out_chantmpls, in_paramtmpls, NULL);

  gui = weed_filter_get_gui(filter_class);
  weed_set_string_value(gui, WEED_LEAF_LAYOUT_SCHEME, "RFX");
  weed_set_string_value(gui, "layout_rfx_delim", "|");
  weed_set_string_array(gui, "layout_rfx_strings", 1, rfx_strings);

  weed_plugin_info_add_filter_class(plugin_info, filter_class);
  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

