/////////////////////////////////////////////////////////////////////////////
// Weed posterise plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script posterise.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////

#define _UNIQUE_ID_ "0XC54D97FB4F78A3C8"

#define NEED_PALETTE_UTILS

#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////

#include <stdio.h>

static int verbosity = WEED_VERBOSITY_ERROR;
enum {
  P_levels,
};

static weed_error_t posterise_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t *in_chan = weed_get_in_channel(inst, 0);
  weed_plant_t *out_chan = weed_get_out_channel(inst, 0);
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  unsigned char *src = weed_channel_get_pixel_data(in_chan);
  unsigned char *dst = weed_channel_get_pixel_data(out_chan);
  int inplace = (src == dst);
  int pal = weed_channel_get_palette(in_chan);
  int irow = weed_channel_get_stride(in_chan);
  int iheight = weed_channel_get_height(in_chan);
  int is_threading = weed_is_threading(inst);
  int offset = 0;
  int width = weed_channel_get_width(out_chan);
  int height = weed_channel_get_height(out_chan);
  int orow = weed_channel_get_stride(out_chan);
  int psize = pixel_size(pal);
  int levels = weed_param_get_value_int(in_params[P_levels]);
  weed_free(in_params);

  if (is_threading) {
    offset = weed_channel_get_offset(out_chan);
    src += offset * irow;
    dst += offset * orow;
  }

  if (1) {
    unsigned char levmask = 128;
    for (int i = 1; i < levels; i++) levmask += 128 >> i;
    width *= psize;
    if (psize == 3) {
      for (int j = 0; j < height; j++) {
        for (int i = 0; i < width; i++) {
          dst[j * orow + i] = src[j * irow + i] & levmask;
        }
      }
    }
    else {
      for (int j = 0; j < height; j++) {
        for (int i = 0; i < width; i += 4) {
          dst[j * orow + i] = src[j * irow + i] & levmask;
          dst[j * orow + i + 1] = src[j * irow + i + 1] & levmask;
          dst[j * orow + i + 2] = src[j * irow + i + 2] & levmask;
          dst[j * orow + i + 3] = src[j * irow + i + 3];
        }
      }
    }
  }

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  uint64_t unique_id;
  int palette_list[] = ALL_RGBX_PALETTES;
  weed_plant_t *in_chantmpls[] = {
      weed_channel_template_init("in_channel0", 0),
      NULL};
  weed_plant_t *out_chantmpls[] = {
      weed_channel_template_init("out_channel0", WEED_CHANNEL_CAN_DO_INPLACE),
      NULL};
  weed_plant_t *in_paramtmpls[] = {
      weed_integer_init("levels", "Colour _levels", 1, 1, 8),
      NULL};
  int filter_flags = WEED_FILTER_HINT_MAY_THREAD;

  verbosity = weed_get_host_verbosity(host_info);

  filter_class = weed_filter_class_init("posterise", "salsaman", 1, filter_flags, palette_list,
    NULL, posterise_process, NULL, in_chantmpls, out_chantmpls, in_paramtmpls, NULL);

  weed_filter_set_description(filter_class, "A simple filter which reduces the number of colour levels in the images.");

  weed_plugin_info_add_filter_class(plugin_info, filter_class);

  if (!sscanf(_UNIQUE_ID_, "0X%lX", &unique_id) || !sscanf(_UNIQUE_ID_, "0x%lx", &unique_id)) {
    weed_set_int64_value(plugin_info, WEED_LEAF_UNIQUE_ID, unique_id);
  }

  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

