/////////////////////////////////////////////////////////////////////////////
// Weed timer plugin, version 1
// Compiled with Builder version 3.2.1-pre
// autogenerated from script timer.script

// released under the GNU GPL version 3 or higher
// see file COPYING or www.gnu.org for details

// (c) salsaman 2021
/////////////////////////////////////////////////////////////////////////////

static int package_version = 1; // version of this package

/////////////////////////////////////////////////////////////////////////////

#define _UNIQUE_ID_ "0XF6B384E9F207FE92"


#ifndef NEED_LOCAL_WEED_PLUGIN
#include <weed/weed-plugin.h>
#ifndef NEED_LOCAL_WEED_UTILS
#include <weed/weed-utils.h> // optional
#else
#include "../../libweed/weed-utils.h" // optional
#endif
#include <weed/weed-plugin-utils.h>
#else
#include "../../libweed/weed-plugin.h"
#include "../../libweed/weed-utils.h" // optional
#include "../../libweed/weed-plugin-utils.h" // optional
#endif

#include "weed-plugin-utils.c"

//////////////////////////////////////////////////////////////////

#include <stdio.h>

static int verbosity = WEED_VERBOSITY_ERROR;

typedef struct {
  weed_timecode_t start, reset;
  int was_started, was_reset;
} sdata_t;

enum {
  P_reset,
};

static weed_error_t timer_init(weed_plant_t *inst) {
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  weed_plant_t **out_params = weed_get_out_params(inst, NULL);
  sdata_t *sdata;

  int reset = weed_param_get_value_boolean(in_params[P_reset]);
  weed_free(in_params);

  if (!(weed_instance_get_flags(inst) & WEED_INSTANCE_UPDATE_GUI_ONLY)) {
    sdata = (sdata_t *)weed_calloc(1, sizeof(sdata_t));
    if (!sdata) return WEED_ERROR_MEMORY_ALLOCATION;
    weed_set_instance_data(inst, sdata);
  }
  else sdata = weed_get_instance_data(inst, sdata);
  weed_free(out_params);

  return WEED_SUCCESS;
}


static weed_error_t timer_deinit(weed_plant_t *inst) {
  sdata_t *sdata = weed_get_instance_data(inst, sdata);

  if (sdata) weed_free(sdata);
  weed_set_instance_data(inst, NULL);

  return WEED_SUCCESS;
}


static weed_error_t timer_process(weed_plant_t *inst, weed_timecode_t tc) {
  weed_plant_t **in_params = weed_get_in_params(inst, NULL);
  weed_plant_t **out_params = weed_get_out_params(inst, NULL);
  sdata_t *sdata = weed_get_instance_data(inst, sdata);
  if (!sdata) return WEED_ERROR_REINIT_NEEDED;
  else {
    int reset = weed_param_get_value_boolean(in_params[P_reset]);
    weed_free(in_params);

    if (1) {
      double tval = (double)tc / (double)WEED_TICKS_PER_SECOND;
        if (!sdata->was_started) {
        sdata->reset = sdata->start = tc;
        sdata->was_started = 1;
      }
      if (reset == WEED_TRUE) {
        if (sdata->was_reset == WEED_FALSE) {
          sdata->reset = tc;
          sdata->was_reset = WEED_TRUE;
        }
      } else sdata->was_reset = WEED_FALSE;

      // absolute
      weed_set_double_value(out_params[1], WEED_LEAF_VALUE, tval);

      // relative
      tval = (double)(tc - sdata->start) / (double)WEED_TICKS_PER_SECOND;
      weed_set_double_value(out_params[0], WEED_LEAF_VALUE, tval);

      // since reset
      tval = (double)(tc - sdata->reset) / (double)WEED_TICKS_PER_SECOND;
      weed_set_double_value(out_params[2], WEED_LEAF_VALUE, tval);

      // was reset (can be fed back to reset)
      weed_set_boolean_value(out_params[3], WEED_LEAF_VALUE, sdata->was_reset);
    }
  weed_free(out_params);
  }

  return WEED_SUCCESS;
}


WEED_SETUP_START(200, 200) {
  weed_plant_t *host_info = weed_get_host_info(plugin_info);
  weed_plant_t *filter_class;
  uint64_t unique_id;
  weed_plant_t *in_paramtmpls[] = {
      weed_switch_init("reset", "_Reset counter", WEED_FALSE),
      NULL};
  weed_plant_t *out_paramtmpls[] = {
      weed_out_param_float_init_nominmax("relative", 0),
      weed_out_param_float_init_nominmax("absolute", 0),
      weed_out_param_float_init_nominmax("since_reset", 0),
      weed_out_param_switch_init("was_reset",  WEED_FALSE),
      NULL};
  int filter_flags = 0;

  verbosity = weed_get_host_verbosity(host_info);

  filter_class = weed_filter_class_init("timer", "salsaman", 1, filter_flags, NULL,
    timer_init, timer_process, timer_deinit, NULL, NULL, in_paramtmpls, out_paramtmpls);

  weed_filter_set_description(filter_class, "Outputs timing values (relative to playback start, absolute, and since reset.\n"
				"The reset value can be set by flipping the 'reset' input from FALSE to TRUE");

  weed_plugin_info_add_filter_class(plugin_info, filter_class);

  if (!sscanf(_UNIQUE_ID_, "0X%lX", &unique_id) || !sscanf(_UNIQUE_ID_, "0x%lx", &unique_id)) {
    weed_set_int64_value(plugin_info, WEED_LEAF_UNIQUE_ID, unique_id);
  }

  weed_plugin_set_package_version(plugin_info, package_version);
}
WEED_SETUP_END;

